"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var debug = require("debug")("raygun").bind(null, "[breadcrumbs]");
var asyncLocalStorage = null;
try {
    asyncLocalStorage = new (require("async_hooks").AsyncLocalStorage)();
    debug("initialized successfully");
}
catch (e) {
    debug("failed to load async_hooks.AsyncLocalStorage - initialization failed\n", e);
}
function returnCallerSite(err, callsites) {
    for (var _i = 0, callsites_1 = callsites; _i < callsites_1.length; _i++) {
        var callsite = callsites_1[_i];
        var fileName = callsite.getFileName() || "";
        if (fileName.startsWith(__dirname)) {
            continue;
        }
        return {
            fileName: fileName,
            functionName: callsite.getFunctionName() || "<anonymous>",
            lineNumber: callsite.getLineNumber(),
        };
    }
    return null;
}
function getCallsite() {
    var originalPrepareStacktrace = Error.prepareStackTrace;
    Error.prepareStackTrace = returnCallerSite;
    var output = {};
    Error.captureStackTrace(output);
    var callsite = output.stack;
    Error.prepareStackTrace = originalPrepareStacktrace;
    return callsite;
}
function addBreadcrumb(breadcrumb, type) {
    if (type === void 0) { type = "manual"; }
    var crumbs = getBreadcrumbs();
    if (!crumbs) {
        return;
    }
    if (typeof breadcrumb === "string") {
        var expandedBreadcrumb = {
            message: breadcrumb,
            level: "info",
            category: "",
        };
        breadcrumb = expandedBreadcrumb;
    }
    var callsite = getCallsite();
    var internalCrumb = __assign(__assign({}, breadcrumb), { category: breadcrumb.category || "", message: breadcrumb.message || "", level: breadcrumb.level || "info", timestamp: Number(new Date()), type: type, className: callsite === null || callsite === void 0 ? void 0 : callsite.fileName, methodName: callsite === null || callsite === void 0 ? void 0 : callsite.functionName, 
        // TODO - do we need to do any source mapping?
        lineNumber: (callsite === null || callsite === void 0 ? void 0 : callsite.lineNumber) || undefined });
    debug("recorded breadcrumb:", internalCrumb);
    crumbs.push(internalCrumb);
}
exports.addBreadcrumb = addBreadcrumb;
function addRequestBreadcrumb(request) {
    var crumbs = getBreadcrumbs();
    if (!crumbs) {
        return;
    }
    var internalCrumb = {
        category: "http",
        message: request.method + " " + request.url,
        level: "info",
        timestamp: Number(new Date()),
        type: "request",
    };
    crumbs.push(internalCrumb);
}
exports.addRequestBreadcrumb = addRequestBreadcrumb;
function getBreadcrumbs() {
    if (!asyncLocalStorage) {
        return null;
    }
    var store = asyncLocalStorage.getStore();
    if (store) {
        return store;
    }
    var newStore = [];
    asyncLocalStorage.enterWith(newStore);
    return newStore;
}
exports.getBreadcrumbs = getBreadcrumbs;
function runWithBreadcrumbs(f) {
    if (!asyncLocalStorage) {
        f();
        return;
    }
    debug("running function with breadcrumbs");
    asyncLocalStorage.run([], f);
}
exports.runWithBreadcrumbs = runWithBreadcrumbs;
var consoleMethods = [
    ["debug", "debug"],
    ["log", "info"],
    ["info", "info"],
    ["warn", "warning"],
    ["error", "error"],
];
var _loop_1 = function (method, level) {
    var oldMethod = console[method];
    console[method] = function logWithBreadcrumb() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        addBreadcrumb({ message: args.join(" "), level: level }, "console");
        return oldMethod.apply(this, args);
    };
};
for (var _i = 0, consoleMethods_1 = consoleMethods; _i < consoleMethods_1.length; _i++) {
    var _a = consoleMethods_1[_i], method = _a[0], level = _a[1];
    _loop_1(method, level);
}
